@@ -32,28 +32,13 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
-#-no-ansi-print-object
 (defmethod print-object ((x meta-info) stream)
   (print-unreadable-object (x stream)
     (format stream "~S ~S, ~D" (meta-info-category x) (meta-info-kind x)
             (meta-info-number x))))
 
-;;; Given any non-negative integer, return a prime number >= to it.
-;;;
-;;; FIXME: This logic should be shared with ALMOST-PRIMIFY in
-;;; hash-table.lisp. Perhaps the merged logic should be
-;;; PRIMIFY-HASH-TABLE-SIZE, implemented as a lookup table of primes
-;;; after integral powers of two:
-;;;    #(17 37 67 131 ..)
-;;; (Or, if that's too coarse, after half-integral powers of two.) By
-;;; thus getting rid of any need for primality testing at runtime, we
-;;; could punt POSITIVE-PRIMEP, too.
-(defun primify (x)
-  (declare (type unsigned-byte x))
-  (do ((n (logior x 1) (+ n 2)))
-      ((positive-primep n) n)))
 
 ;;; Why do we suppress the :COMPILE-TOPLEVEL situation here when we're
 ;;; running the cross-compiler? The cross-compiler (which was built
@@ -87,23 +72,6 @@
            (!make-meta-info id category kind type-spec type-checker
                             validate-function default)))))
 
-#-sb-xc
-(setf (get '!%define-info-type :sb-cold-funcall-handler/for-effect)
-      (lambda (category kind type-spec checker validator default id)
-        ;; The SB!FASL: symbols are poor style, but the lesser evil.
-        ;; If exported, then they'll stick around in the target image.
-        ;; Perhaps SB-COLD should re-export some of these.
-        (declare (special sb!fasl::*dynamic* sb!fasl::*cold-layouts*))
-        (let ((layout (gethash 'meta-info sb!fasl::*cold-layouts*)))
-          (sb!fasl::cold-svset
-           (sb!fasl::cold-symbol-value '*info-types*)
-           id
-           (sb!fasl::write-slots
-            (sb!fasl::allocate-struct sb!fasl::*dynamic* layout)
-            'meta-info ; give the type name in lieu of layout
-            :category category :kind kind :type-spec type-spec
-            :type-checker checker :validate-function validator
-            :default default :number id)))))
 
 ;;;; info types, and type numbers, part II: what's
 ;;;; needed only at compile time, not at run time
@@ -124,29 +92,27 @@
 ;;;  :DEFAULT (CONSTANTLY #'<a-function-name>) to adhere to the convention
 ;;; that default objects satisfying FUNCTIONP will always be funcalled.
 ;;;
-(eval-when (:compile-toplevel :execute)
-;; This convoluted idiom creates a macro that disappears from the target,
-;; kind of an alternative to the "!" name convention.
-(#+sb-xc-host defmacro
- #-sb-xc-host sb!xc:defmacro
-    define-info-type ((category kind)
-                       &key (type-spec (missing-arg))
-                            (validate-function)
-                            default)
+(defmacro define-info-type ((category kind)
+                            &key (type-spec (missing-arg))
+                                 (validate-function)
+                                 default)
   (declare (type keyword category kind))
   ;; There was formerly a remark that (COPY-TREE TYPE-SPEC) ensures repeatable
   ;; fasls. That's not true now, probably never was. A compiler is permitted to
   ;; coalesce EQUAL quoted lists and there's no defense against it, so why try?
-  `(!%define-info-type
-           ,category ,kind ',type-spec
-           ,(if (eq type-spec 't)
-                '#'identity
-                `(named-lambda "check-type" (x) (the ,type-spec x)))
-           ,validate-function ,default
-           ;; Rationale for hardcoding here is explained at INFO-VECTOR-FDEFN.
-           ,(or (and (eq category :function) (eq kind :definition)
-                     +fdefn-info-num+)
-                #+sb-xc (meta-info-number (meta-info category kind))))))
+  `(!cold-init-forms
+    (!%define-info-type
+     ,category ,kind ',type-spec
+     ,(if (eq type-spec 't)
+          '#'identity
+          `(named-lambda "check-type" (x) (the ,type-spec x)))
+     ,validate-function ,default
+     ;; Rationale for hardcoding here is explained at INFO-VECTOR-FDEFN.
+     ,(or (and (eq category :function) (eq kind :definition)
+               +fdefn-info-num+)
+          #+sb-xc (meta-info-number (meta-info category kind))))))
+;; It's an external symbol of SB-INT so wouldn't be removed automatically
+(push '("SB-INT" define-info-type) *!removable-symbols*)
 
 
 (macrolet ((meta-info-or-lose (category kind)
@@ -185,6 +151,8 @@
   ;; to be certain that it's not supposed to happen when building the xc.
   #+sb-xc-xhost (error "Strange CLEAR-INFO building the xc: ~S ~S"
                        name info-numbers)
+  (when (pcl-methodfn-name-p name)
+    (error "Can't SET-INFO-VALUE on PCL-internal function"))
   (let (new)
     (with-globaldb-name (key1 key2) name
       :simple
@@ -204,30 +172,28 @@
                   (if old
                       ;; if -REMOVE => nil, then update NEW but return OLD
                       (or (setq new (packed-info-remove
-                                     old +no-auxilliary-key+ info-numbers))
+                                     old +no-auxiliary-key+ info-numbers))
                           old))))
         (info-puthash *info-environment* name #'clear-hairy)))
     (not (null new))))
 
-;;;; *INFO-ENVIRONMENT*
-
-(defun !globaldb-cold-init ()
-  ;; Genesis writes the *INFO-TYPES* array, but setting up the mapping
-  ;; from keyword-pair to object is deferred until cold-init.
-  (dovector (x (the simple-vector *info-types*))
-    (when x (!register-meta-info x)))
-  (setq *info-environment* (make-info-hashtable)))
-
 ;;;; GET-INFO-VALUE
 
 ;;; If non-nil, *GLOBALDB-OBSERVER*'s CAR is a bitmask over info numbers
 ;;; for which you'd like to call the function in the CDR whenever info
 ;;; of that number is queried.
-(!defvar *globaldb-observer* nil)
+(defvar *globaldb-observer* nil)
 (declaim (type (or (cons (unsigned-byte #.(ash 1 info-number-bits)) function)
                    null) *globaldb-observer*))
 #-sb-xc-host (declaim (always-bound *globaldb-observer*))
 
+#+sb-xc-host
+(progn
+  (defun info-gethash (key table) (gethash key table))
+  (defun info-puthash (table key augmenter)
+    (let ((old (gethash key table)))
+      (setf (gethash key table) (funcall augmenter old)))))
+
 ;;; Return the value of NAME / INFO-NUMBER from the global environment,
 ;;; or return the default if there is no global info.
 ;;; The secondary value indicates whether info was found vs defaulted.
@@ -245,7 +211,7 @@
            ;; but constraint propagation isn't informing the compiler here.
            :simple (values (symbol-info-vector (truly-the symbol key1)) key2)
            :hairy (values (info-gethash name *info-environment*)
-                          +no-auxilliary-key+)))
+                          +no-auxiliary-key+)))
       (when vector
         (let ((index (packed-info-value-index vector aux-key info-number)))
           (when index
@@ -259,31 +225,10 @@
       (when hookp
         (funcall (truly-the function (cdr hook)) name info-number answer nil))
       (values answer nil))))
-
-;; Call FUNCTION once for each Name in globaldb that has information associated
-;; with it, passing the function the Name as its only argument.
-;;
-(defun call-with-each-globaldb-name (fun-designator)
-  (let ((function (coerce fun-designator 'function)))
-    (with-package-iterator (iter (list-all-packages) :internal :external)
-      (loop (multiple-value-bind (winp symbol access package) (iter)
-              (declare (ignore access))
-              (if (not winp) (return))
-              ;; Try to process each symbol at most once by associating it with
-              ;; a single package. If a symbol is apparently uninterned,
-              ;; always keep it since we can't know if it has been seen once.
-              (when (or (not (symbol-package symbol))
-                        (eq package (symbol-package symbol)))
-                (dolist (name (info-vector-name-list symbol))
-                  (funcall function name))))))
-    (info-maphash (lambda (name data)
-                    (declare (ignore data))
-                    (funcall function name))
-                  *info-environment*)))
 
+(!begin-collecting-cold-init-forms)
 ;;;; ":FUNCTION" subsection - Data pertaining to globally known functions.
-
-(define-info-type (:function :definition) :type-spec (or fdefn null))
+(define-info-type (:function :definition) :type-spec #-sb-xc-host (or fdefn null) #+sb-xc-host t)
 
 ;;; the kind of functional object being described. If null, NAME isn't
 ;;; a known functional object.
@@ -297,17 +242,42 @@
   ;; 19990330
   :default
   #+sb-xc-host nil
-  #-sb-xc-host (lambda (name) (if (fboundp name) :function nil)))
+  #-sb-xc-host (lambda (name)
+                 (if (or (fboundp name) (pcl-methodfn-name-p name))
+                     :function
+                     nil)))
+
+;;; The deferred mode processor for fasteval special operators.
+;;; Immediate processors are hung directly off symbols in a dedicated slot.
+#+sb-fasteval
+(define-info-type (:function :interpreter) :type-spec (or function null))
 
 ;;; Indicates whether the function is deprecated.
 (define-info-type (:function :deprecated)
   :type-spec (or null deprecation-info))
 
-(declaim (ftype (sfunction (t) ctype)
-                specifier-type ctype-of sb!kernel::ctype-of-array))
+;;; FIXME: Why are these here? It seems like the wrong place.
+(declaim (ftype (sfunction (t &optional t symbol) ctype) specifier-type)
+         (ftype (sfunction (t) ctype) ctype-of sb-kernel::ctype-of-array))
+
+;;; The parsed or unparsed type for this function, or the symbol :GENERIC-FUNCTION.
+;;; Ordinarily a parsed type is stored. Only if the parsed type contains
+;;; an unknown type will the original specifier be stored; we attempt to reparse
+;;; on each lookup, in the hope that the type becomes known at some point.
+;;; If :GENERIC-FUNCTION, the info is recomputed from methods at the time of lookup
+;;; and stored back. Method redefinition resets the value to :GENERIC-FUNCTION.
+(define-info-type (:function :type)
+  :type-spec (or ctype (cons (eql function)) (member :generic-function))
+  :default (lambda (name)
+             (declare (ignorable name))
+             #+sb-xc-host (specifier-type 'function)
+             #-sb-xc-host (sb-impl::ftype-from-fdefn name)))
 
 ;;; the ASSUMED-TYPE for this function, if we have to infer the type
 ;;; due to not having a declaration or definition
+;;; FIXME: It may be better to have this and/or :TYPE stored in a single
+;;; property as either (:known . #<ctype>) or (:assumed . #<ctype>)
+;;; rather than using two different properties. Do we ever use *both* ?
 (define-info-type (:function :assumed-type)
   ;; FIXME: The type-spec really should be
   ;;   (or approximate-fun-type null)).
@@ -335,17 +305,21 @@
   #+sb-xc-host :assumed
   #-sb-xc-host (lambda (name) (if (fboundp name) :defined :assumed)))
 
-;;; something which can be decoded into the inline expansion of the
-;;; function, or NIL if there is none
-;;;
-;;; To inline a function, we want a lambda expression, e.g.
-;;; '(LAMBDA (X) (+ X 1)).
-(define-info-type (:function :inline-expansion-designator)
-  :type-spec list)
+;;; Two kinds of hints for compiling calls as efficiently as possible:
+;;; (A) Inline expansion: To inline a function, we want a lambda
+;;; expression, e.g. '(LAMBDA (X) (+ X 1)) or a lambda-with-lexenv.
+;;; (B) List of arguments which could be dynamic-extent closures, and which
+;;; we could, under suitable compilation policy, DXify in the caller
+;;; especially when open-coding a call to this function.
+;;; If only (A) is stored, then this value is a list (the lambda expression).
+;;; If only (B) is stored, then this is a DXABLE-ARGS.
+;;; If both, this is an INLINING-DATA.
+(define-info-type (:function :inlining-data)
+  :type-spec (or list sb-c::dxable-args sb-c::inlining-data))
 
 ;;; This specifies whether this function may be expanded inline. If
 ;;; null, we don't care.
-(define-info-type (:function :inlinep) :type-spec sb!c::inlinep)
+(define-info-type (:function :inlinep) :type-spec sb-c::inlinep)
 
 ;;; Track how many times IR2 converted a call to this function as a full call
 ;;; that was not in the scope of a local or global notinline declaration.
@@ -353,6 +327,22 @@
 ;;; through some kind of transformation but were not.
 (define-info-type (:function :emitted-full-calls) :type-spec list)
 
+;; Return the number of calls to NAME that IR2 emitted as full calls,
+;; not counting calls via #'F that went untracked.
+;; Return 0 if the answer is nonzero but a warning was already signaled
+;; about any full calls were emitted. This return convention satisfies the
+;; intended use of this statistic - to decide whether to generate a warning
+;; about failure to inline NAME, which is shown at most once per name
+;; to avoid unleashing a flood of identical warnings.
+(defun emitted-full-call-count (name)
+  (let ((status (car (info :function :emitted-full-calls name))))
+     (and (integerp status)
+          ;; Bit 0 tells whether any call was NOT in the presence of
+          ;; a 'notinline' declaration, thus eligible to be inline.
+          ;; Bit 1 tells whether any warning was emitted yet.
+          (= (logand status 3) #b01)
+          (ash status -2)))) ; the call count as tracked by IR2
+
 ;;; a macro-like function which transforms a call to this function
 ;;; into some other Lisp form. This expansion is inhibited if inline
 ;;; expansion is inhibited.
@@ -365,6 +355,8 @@
 ;;; We don't actually have anything like that any more though.
 ;;; For user-defined functions, the invariant is maintained that at most
 ;;; one of :source-transform and an inline-expansion exist.
+;;; However, there is one exception: a structure constructor can have an
+;;; inline expansion and also store (#<dd> . :constructor) here.
 (define-info-type (:function :source-transform)
   :type-spec (or function null (cons atom atom)))
 
@@ -380,7 +372,14 @@
 
 ;;; If a function is "known" to the compiler, then this is a FUN-INFO
 ;;; structure containing the info used to special-case compilation.
-(define-info-type (:function :info) :type-spec (or sb!c::fun-info null))
+(define-info-type (:function :info) :type-spec (or sb-c::fun-info null))
+
+;;; For PCL code walker
+(define-info-type (:function :walker-template) :type-spec (or list symbol))
+
+;;; The exact type for which this function is the predicate.
+;;; Applicable only for symbols in the CL package.
+(define-info-type (:function :predicate-for) :type-spec t)
 
 ;;; This is a type specifier <t> such that if an argument X to the function
 ;;; does not satisfy (TYPEP x <t>) then the function definitely returns NIL.
@@ -423,15 +422,14 @@
 (define-info-type (:variable :macro-expansion) :type-spec t)
 
 (define-info-type (:variable :alien-info)
-  :type-spec (or null sb!alien-internals:heap-alien-info))
+  :type-spec (or null sb-alien-internals:heap-alien-info))
 
 (define-info-type (:variable :documentation) :type-spec (or string null))
 
 ;; :WIRED-TLS describes how SYMBOL-VALUE (implicit or not) should be compiled.
-;;  - :ALWAYS-HAS-TLS means that calls to SYMBOL-VALUE should access the TLS
-;;     with a fixed offset. The index is assigned no later than load-time of
-;;     the file containing code thus compiled. Presence of an index in the
-;;     image that performed compilation is irrelevant (for now).
+;;  - T means that SYMBOL-VALUE should access the TLS with a fixed offset,
+;;     resolved at load-time. Presence of an index in the image that performed
+;;     compilation is irrelevant (for now).
 ;;  - :ALWAYS-THREAD-LOCAL implies a fixed offset, *and* that the check for
 ;;     no-tls-value may be elided. There is currently no way to set this.
 ;;     Note that this does not affect elision of the check for unbound-marker
@@ -447,24 +445,9 @@
 ;; ever referenced by SYMBOL-VALUE or SET. Depletion should occur lazily.
 ;;
 (define-info-type (:variable :wired-tls)
-    :type-spec (or (member nil :always-has-tls :always-thread-local)
-                   fixnum) ; the actual index, for thread slots (to be done)
-    :default
-    (lambda (symbol)
-      (declare (symbol symbol))
-      (and (eq (info :variable :kind symbol) :special)
-           #-sb-xc-host
-           (eq (symbol-package symbol) *cl-package*)
-           #+sb-xc-host
-           (flet ((external-in-package-p (pkg)
-                    (and (string= (package-name (symbol-package symbol)) pkg)
-                         (eq (nth-value 1 (find-symbol (string symbol) pkg))
-                             :external))))
-             ;; I'm not worried about random extra externals in some bizarro
-             ;; host lisp. TLS assignment has no bearing on semantics at all.
-             (or (external-in-package-p "COMMON-LISP")
-                 (external-in-package-p "SB-XC")))
-           :always-has-tls)))
+    :type-spec (or (member nil t :always-thread-local)
+                   fixnum) ; the actual index, for thread slots
+    :default nil)
 
 ;;;; ":TYPE" subsection - Data pertaining to globally known types.
 
@@ -482,7 +465,7 @@
                        ;; The compiler-macro signals an error
                        ;; on forward-referenced info-types.
                        #+sb-xc-host (declare (notinline info))
-                       (when (info :declaration :recognized name)
+                       (when (info :declaration :known name)
                          (error 'declaration-type-conflict-error
                                 :format-arguments (list name)))))
 
@@ -502,6 +485,13 @@
 ;;; The classoid-cell for this type
 (define-info-type (:type :classoid-cell) :type-spec t)
 
+;;; wrapper for this type being used by the compiler
+(define-info-type (:type :compiler-layout)
+  :type-spec (or wrapper null)
+  :default (lambda (name)
+             (let ((class (find-classoid name nil)))
+               (and class (classoid-wrapper class)))))
+
 ;;; DEFTYPE lambda-list
 ;; FIXME: remove this after making swank-fancy-inspector not use it.
 (define-info-type (:type :lambda-list) :type-spec t)
@@ -524,8 +514,8 @@
 ;; Therefore maintain a list of recognized declarations. This list makes the
 ;; globaldb storage of same redundant, but oh well.
 (defglobal *recognized-declarations* nil)
-(define-info-type (:declaration :recognized)
-  :type-spec boolean
+(define-info-type (:declaration :known)
+  :type-spec (or function boolean)
   ;; There's no portable way to unproclaim that a symbol is a declaration,
   ;; but at the low-level permit new-value to be NIL.
   :validate-function (lambda (name new-value)
@@ -539,26 +529,22 @@
                               (setq *recognized-declarations*
                                     (delete name *recognized-declarations*))))))
 
-(define-info-type (:declaration :handler) :type-spec (or function null))
-
 ;;;; ":ALIEN-TYPE" subsection - Data pertaining to globally known alien-types.
 (define-info-type (:alien-type :kind)
   :type-spec (member :primitive :defined :unknown)
   :default :unknown)
 (define-info-type (:alien-type :translator) :type-spec (or function null))
 (define-info-type (:alien-type :definition)
-  :type-spec (or null sb!alien-internals:alien-type))
+  :type-spec (or null sb-alien-internals:alien-type))
 (define-info-type (:alien-type :struct)
-  :type-spec (or null sb!alien-internals:alien-type))
+  :type-spec (or null sb-alien-internals:alien-type))
 (define-info-type (:alien-type :union)
-  :type-spec (or null sb!alien-internals:alien-type))
+  :type-spec (or null sb-alien-internals:alien-type))
 (define-info-type (:alien-type :enum)
-  :type-spec (or null sb!alien-internals:alien-type))
+  :type-spec (or null sb-alien-internals:alien-type))
 
 ;;;; ":SETF" subsection - Data pertaining to expansion of the omnipotent macro.
-(define-info-type (:setf :documentation) :type-spec (or string null))
-(define-info-type (:setf :expander)
-    :type-spec (or symbol function (cons integer function) null))
+(define-info-type (:setf :expander) :type-spec (or function list))
 
 ;;;; ":CAS" subsection - Like SETF but there are no "inverses", just expanders
 (define-info-type (:cas :expander) :type-spec (or function null))
@@ -581,10 +567,20 @@
 (define-info-type (:source-location :constant) :type-spec t)
 (define-info-type (:source-location :typed-structure) :type-spec t)
 (define-info-type (:source-location :symbol-macro) :type-spec t)
-(define-info-type (:source-location :vop) :type-spec t)
 (define-info-type (:source-location :declaration) :type-spec t)
 (define-info-type (:source-location :alien-type) :type-spec t)
 
+(!defun-from-collected-cold-init-forms !info-type-cold-init)
+
+#-sb-xc-host
+(defun !globaldb-cold-init ()
+  (let ((h (make-info-hashtable)))
+    (setf (sb-thread:mutex-name (info-env-mutex h)) "globaldb")
+    (setq *info-environment* h))
+  (setq *globaldb-observer* nil)
+  (setq *info-types* (make-array (ash 1 info-number-bits) :initial-element nil))
+  (!info-type-cold-init))
+
 ;; This is for the SB-INTROSPECT contrib module, and debugging.
 (defun call-with-each-info (function symbol)
   (awhen (symbol-info-vector symbol)
@@ -602,5 +598,5 @@
                  (if (not type) type-num)
                  (if type
                      (list (meta-info-category type) (meta-info-kind type))))
-         (write val :level 1)))
+         (write val :level 2)))
      sym)))
