@@ -20,55 +20,10 @@
 ;;;; Note: Take a look at the compiler-overview.tex section on "Hairy
 ;;;; function representation" before you seriously mess with this
 ;;;; stuff.
-
-(declaim (ftype (sfunction * function) make-repeated-name-check))
-(defun make-repeated-name-check (&key
-                                   (kind "variable")
-                                   (context "lambda list")
-                                   (signal-via #'compiler-error))
-  (let ((seen '()))
-    (lambda (name)
-      (when (member name seen :test #'eq)
-        (funcall signal-via "~@<The ~A ~S occurs more than once in ~
-                             the ~A.~@:>"
-                 kind name context))
-      (push name seen)
-      name)))
-
-;;; Verify that NAME is a legal name for a variable.
-(declaim (ftype (function (t &key
-                             (:context t) (:allow-special t) (:allow-symbol-macro t)
-                             (:signal-via (or symbol function)))
-                          (values symbol keyword))
-                check-variable-name-for-binding))
-(defun check-variable-name-for-binding (name
-                                        &key
-                                          context
-                                          (allow-special t)
-                                          (allow-symbol-macro t)
-                                          (signal-via #'compiler-error))
-  (check-variable-name name :signal-via signal-via)
-  (flet ((lose (kind)
-           (funcall signal-via
-                    (sb-format:tokens "~@<~/sb-ext:print-symbol-with-prefix/ names a ~
-                               ~A, and cannot be used in ~A.~:@>")
-                    name kind context)))
-    (let ((kind (info :variable :kind name)))
-      (case kind
-        (:macro
-         (unless allow-symbol-macro
-           (program-assert-symbol-home-package-unlocked
-            :compile name (format nil "lexically binding global ~
-                                       symbol-macro ~~A in ~A"
-                                  context))))
-        ((:constant)
-         (lose "defined constant"))
-        ((:global)
-         (lose "global lexical variable"))
-        (:special
-         (unless allow-special
-           (lose "special variable"))))
-      (values name kind))))
+#-sb-devel
+(declaim (start-block ir1-convert-lambda ir1-convert-lambda-body
+                      ir1-convert-aux-bindings varify-lambda-arg
+                      ir1-convert-lambdalike))
 
 ;;; Return a VAR structure for NAME, filling in info if it is globally
 ;;; special. If it is losing, we punt with a COMPILER-ERROR.
@@ -112,7 +67,8 @@
              (add-info (var kind &key (default nil defaultp) suppliedp-var key)
                (let ((info (make-arg-info :kind kind)))
                  (when defaultp
-                   (setf (arg-info-default info) default))
+                   (setf (arg-info-default info) default
+                         (arg-info-default-p info) t))
                  (when suppliedp-var
                    (setf (arg-info-supplied-p info)
                          (varify-lambda-arg suppliedp-var)))
@@ -259,11 +215,6 @@
                   :system-lambda-p system-lambda))
          (result-ctran (make-ctran))
          (result-lvar (make-lvar)))
-
-    (awhen (lexenv-lambda *lexenv*)
-      (push lambda (lambda-children it))
-      (setf (lambda-parent lambda) it))
-
     ;; just to check: This function should fail internal assertions if
     ;; we didn't set up a valid debug name above.
     ;;
@@ -354,7 +305,7 @@
          (fun (collect ((default-bindings)
                         (default-vals))
                 (dolist (default defaults)
-                  (if (sb-xc:constantp default)
+                  (if (constantp default)
                       (default-vals default)
                       (let ((var (sb-xc:gensym)))
                         (default-bindings `(,var ,default))
@@ -408,7 +359,7 @@
          (default (arg-info-default info))
          (supplied-p (arg-info-supplied-p info))
          (force (or force
-                    (not (sb-xc:constantp (arg-info-default info)))))
+                    (not (constantp (arg-info-default info)))))
          (ep (if supplied-p
                  (ir1-convert-hairy-args
                   res
@@ -529,7 +480,9 @@
               (found-allow-p nil))
 
           (temps #-stack-grows-downward-not-upward
-                 `(,n-index (1- ,n-count))
+                 `(,n-index (+ ,n-count ,(if (vop-existsp :translate %more-keyword-pair)
+                                             0
+                                             -1)))
                  #+stack-grows-downward-not-upward
                  `(,n-index (- (1- ,n-count)))
                  #-stack-grows-downward-not-upward n-value-temp
@@ -585,7 +538,9 @@
                        (setq ,n-lose ,n-key))))
 
             (body
-             `(when (oddp ,n-count)
+             `(when (oddp ,(if (vop-existsp :translate %more-keyword-pair)
+                               n-index
+                               n-count))
                 (%odd-key-args-error)))
 
             (body
@@ -593,12 +548,19 @@
              `(locally
                 (declare (optimize (safety 0)))
                 (loop
-                  (when (minusp ,n-index) (return))
-                  (setf ,n-value-temp (%more-arg ,n-context ,n-index))
-                  (decf ,n-index)
-                  (setq ,n-key (%more-arg ,n-context ,n-index))
-                  (decf ,n-index)
-                  (case ,n-key ,@(tests))))
+                 ,@(cond ((vop-existsp :translate %more-keyword-pair)
+                          `((when (zerop ,n-index) (return))
+                            (decf ,n-index 2)
+                            (multiple-value-bind (key value)
+                                (%more-keyword-pair ,n-context ,n-index)
+                              (setf ,n-value-temp value ,n-key key))))
+                         (t
+                          `((when (minusp ,n-index) (return))
+                            (setf ,n-value-temp (%more-arg ,n-context ,n-index))
+                            (decf ,n-index)
+                            (setq ,n-key (%more-arg ,n-context ,n-index))
+                            (decf ,n-index))))
+                 (case ,n-key ,@(tests))))
              #+stack-grows-downward-not-upward
              `(locally (declare (optimize (safety 0)))
                 (loop
@@ -610,7 +572,7 @@
                     (case ,n-key ,@(tests))))))
 
             (unless allowp
-              (let ((location (make-restart-location)))
+              (let ((location (opaquely-quote (make-restart-location))))
                 (body `(if (and (not (unbound-marker-p ,n-lose))
                                 (not ,n-allowp))
                            (%unknown-key-arg-error ,n-lose ,location)
@@ -686,7 +648,7 @@
     (dolist (key keys)
       (let* ((info (lambda-var-arg-info key))
              (default (arg-info-default info))
-             (hairy-default (not (sb-xc:constantp default)))
+             (hairy-default (not (constantp default)))
              (supplied-p (arg-info-supplied-p info))
              ;; was: (format nil "~A-DEFAULTING-TEMP" (leaf-source-name key))
              (n-val (make-symbol ".DEFAULTING-TEMP."))
@@ -891,9 +853,7 @@
                                      :keyp keyp
                                      :%source-name source-name
                                      :%debug-name debug-name
-                                     :plist `(:ir1-environment
-                                              (,*lexenv*
-                                               ,*current-path*))))
+                                     :source-path *current-path*))
         (min (or (position-if #'lambda-var-arg-info vars) (length vars))))
     (aver-live-component *current-component*)
     (ir1-convert-hairy-args res () () () () vars nil body aux-vars aux-vals
@@ -935,7 +895,7 @@
       (multiple-value-bind (vars keyp allow-other-keys aux-vars aux-vals)
           (make-lambda-vars (cadr form))
         (binding* (((*lexenv* result-type post-binding-lexenv
-                              lambda-list explicit-check)
+                              lambda-list explicit-check source-form)
                     (process-decls decls (append aux-vars vars) nil
                                    :binding-form-p t :allow-lambda-list t))
                    (debug-catch-p (and maybe-add-debug-catch
@@ -967,7 +927,7 @@
                                                         :system-lambda system-lambda)))))
           (when explicit-check
             (setf (getf (functional-plist res) 'explicit-check) explicit-check))
-          (setf (functional-inline-expansion res) form)
+          (setf (functional-inline-expansion res) (or source-form form))
           (setf (functional-arg-documentation res)
                 (if (eq lambda-list :unspecified)
                     (strip-lambda-list (cadr form) :arglist)
@@ -1044,8 +1004,12 @@
                                           :maybe-add-debug-catch t
                                           :source-name name))
                  (info (info :function :info name)))
+             (setf (functional-inlinep res) (info :function :inlinep name)
+                   (defined-fun-named-lambda-p defined-fun-res) t)
              (when (has-toplevelness-decl lambda-expression)
                (setf (functional-top-level-defun-p res) t))
+             ;; FIXME: Should non-entry block compiled defuns have
+             ;; this propagate?
              (assert-global-function-definition-type name res)
              (push res (defined-fun-functionals defined-fun-res))
              (unless (or
@@ -1061,10 +1025,12 @@
                                (fun-info-ltn-annotate info)
                                (fun-info-ir2-convert info)
                                (fun-info-optimizer info))))
-               (substitute-leaf-if
-                (lambda (ref)
-                  (policy ref (> recognize-self-calls 0)))
-                res defined-fun-res))
+               (if (block-compile *compilation*)
+                   (substitute-leaf res defined-fun-res)
+                   (substitute-leaf-if
+                    (lambda (ref)
+                      (policy ref (> recognize-self-calls 0)))
+                    res defined-fun-res)))
              res)
            (ir1-convert-lambda lambda-expression
                                :maybe-add-debug-catch t
@@ -1074,7 +1040,150 @@
      (ir1-convert-inline-lambda thing
                                 :source-name source-name
                                 :debug-name debug-name))))
+
+(declaim (end-block))
 
+
+;;;; defining global functions
+;;; Given a lambda-list, return a FUN-TYPE object representing the signature:
+;;; return type is *, and each individual arguments type is T -- but we get
+;;; the argument counts and keywords.
+;;; TODO: enhance this to optionally accept an alist of (var . type)
+;;; and use that lieu of SB-INTERPRETER:APPROXIMATE-PROTO-FN-TYPE.
+(defun ftype-from-lambda-list (lambda-list)
+  (multiple-value-bind (llks req opt rest key-list)
+      (parse-lambda-list lambda-list :silent t)
+    (flet ((list-of-t (list) (mapcar (constantly t) list)))
+      (let ((reqs (list-of-t req))
+            (opts (when opt (cons '&optional (list-of-t opt))))
+            ;; When it comes to building a type, &REST means pretty much the
+            ;; same thing as &MORE.
+            (rest (when rest '(&rest t)))
+            (keys (when (ll-kwds-keyp llks)
+                    (cons '&key (mapcar (lambda (spec)
+                                          (list (parse-key-arg-spec spec) t))
+                                        key-list))))
+            (allow (when (ll-kwds-allowp llks) '(&allow-other-keys))))
+        (careful-specifier-type `(function (,@reqs ,@opts ,@rest ,@keys ,@allow) *))))))
+
+#-sb-devel
+(declaim (start-block maybe-inline-syntactic-closure))
+
+;;; Take the lexenv surrounding an inlined function and extract things
+;;; needed for the inline expansion suitable for dumping into fasls.
+;;; Right now it's MACROLET, SYMBOL-MACROLET, SPECIAL and
+;;; INLINE/NOTINLINE declarations. Upon encountering something else return NIL.
+;;; This is later used by PROCESS-INLINE-LEXENV to reproduce the lexenv.
+;;;
+;;; Previously it just used the functions and vars of the innermost
+;;; lexenv, but the body of macrolet can refer to other macrolets
+;;; defined earlier, so it needs to process all the parent lexenvs to
+;;; recover the proper order.
+(defun reconstruct-lexenv (lexenv)
+  (let (shadowed-funs
+        shadowed-vars
+        result)
+    (loop for env = lexenv then parent
+          for parent = (lexenv-parent env)
+          for vars = (lexenv-vars env)
+          for funs = (lexenv-funs env)
+          for declarations = nil
+          for symbol-macros = nil
+          for macros = nil
+          do
+          (loop for binding in vars
+                for (name . what) = binding
+                unless (and parent
+                            (find binding (lexenv-vars parent)))
+                do (typecase what
+                     (cons
+                      (aver (eq (car what) 'macro))
+                      (push name shadowed-vars)
+                      (push (list name (cdr what)) symbol-macros))
+                     (global-var
+                      (aver (eq (global-var-kind what) :special))
+                      (push `(special ,name) declarations))
+                     (t
+                      (unless (memq name shadowed-vars)
+                        (return-from reconstruct-lexenv)))))
+          (loop for binding in funs
+                for (name . what) = binding
+                unless (and parent
+                            (find binding (lexenv-funs parent)))
+                do
+                (typecase what
+                  (cons
+                   (push name shadowed-funs)
+                   (let ((expression (function-lambda-expression (cdr what))))
+                     (aver expression)
+                     (push (cons name expression) macros)))
+                  ;; FIXME: Is there a good reason for this not to be
+                  ;; DEFINED-FUN (which :INCLUDEs GLOBAL-VAR, in case
+                  ;; you're wondering how this ever worked :-)? Maybe
+                  ;; in conjunction with an AVERrance that it's not an
+                  ;; (AND GLOBAL-VAR (NOT GLOBAL-FUN))? -- CSR,
+                  ;; 2002-07-08
+                  (global-var
+                   (unless (defined-fun-p what)
+                     (return-from reconstruct-lexenv))
+                   (push `(,(car (defined-fun-inlinep what))
+                           ,name)
+                         declarations))
+                  (t
+                   (unless (memq name shadowed-funs)
+                     (return-from reconstruct-lexenv)))))
+          (when declarations
+            (setf result (list* :declare declarations (and result (list result)))))
+          (when symbol-macros
+            (setf result (list* :symbol-macro symbol-macros (and result (list result)))))
+          (when macros
+            (setf result (list* :macro macros (and result (list result)))))
+          while (and parent
+                     (not (null-lexenv-p parent))))
+    result))
+
+;;; Return a sexpr for LAMBDA in LEXENV such that loading it from fasl
+;;; preserves the original lexical environment for inlining.
+;;; Return NIL if the lexical environment is too complicated.
+(defun maybe-inline-syntactic-closure (lambda lexenv)
+  (declare (type list lambda) (type lexenv-designator lexenv))
+  (aver (eql (first lambda) 'lambda))
+  ;; We used to have a trivial implementation, verifying that lexenv
+  ;; was effectively null. However, this fails to take account of the
+  ;; idiom
+  ;;
+  ;; (declaim (inline foo))
+  ;; (macrolet ((def (x) `(defun ,x () ...)))
+  ;;   (def foo))
+  ;;
+  ;; which, while too complicated for the cross-compiler to handle in
+  ;; unfriendly foreign lisp environments, would be good to support in
+  ;; the target compiler. -- CSR, 2002-05-13 and 2002-11-02
+  (typecase lexenv
+   (lexenv
+    (let ((vars (lexenv-vars lexenv))
+          (funs (lexenv-funs lexenv)))
+      (acond ((or (lexenv-blocks lexenv) (lexenv-tags lexenv)) nil)
+             ((and (null vars) (null funs)) lambda)
+             ;; If the lexenv is too hairy for cross-compilation,
+             ;; you'll find out later, when trying to perform inlining.
+             ;; This is fine, because if the inline expansion is only
+             ;; for the target, it's totally OK to cross-compile this
+             ;; defining form. The syntactic env is correctly captured.
+             ((reconstruct-lexenv lexenv)
+              `(lambda-with-lexenv ,it ,@(cdr lambda))))))
+   #+(and sb-fasteval (not sb-xc-host))
+   (sb-interpreter:basic-env
+    (awhen (sb-interpreter::reconstruct-syntactic-closure-env lexenv)
+      `(lambda-with-lexenv ,it ,@(cdr lambda))))
+   #+sb-fasteval
+   (null lambda))) ; trivial case. Never occurs in the compiler.
+
+(declaim (end-block))
+
+#-sb-devel
+(declaim (start-block ir1-convert-inline-lambda))
+
 ;;; Convert the forms produced by RECONSTRUCT-LEXENV to LEXENV
 (defun process-inline-lexenv (inline-lexenv)
   (labels ((recurse (inline-lexenv lexenv)
@@ -1122,78 +1231,72 @@
          (body (if lambda-with-lexenv-p
                    `(lambda ,@(cddr fun))
                    fun))
+         (lexenv-lambda (lexenv-lambda *lexenv*))
          (*lexenv*
            (if lambda-with-lexenv-p
                (make-lexenv
                 :default (process-inline-lexenv (second fun))
                 :handled-conditions (lexenv-handled-conditions *lexenv*)
                 :policy policy
-                :flushable (lexenv-flushable *lexenv*))
+                :flushable (lexenv-flushable *lexenv*)
+                :lambda lexenv-lambda
+                :parent *lexenv*)
                (make-almost-null-lexenv
                 policy
                 ;; Inherit MUFFLE-CONDITIONS from the call-site lexenv
                 ;; rather than the definition-site lexenv, since it seems
                 ;; like a much more common case.
                 (lexenv-handled-conditions *lexenv*)
-                (lexenv-flushable *lexenv*))))
+                (lexenv-flushable *lexenv*)
+                lexenv-lambda
+                *lexenv*)))
+         (*inlining* (not *transforming*))
          (clambda (ir1-convert-lambda body
                                       :source-name source-name
                                       :debug-name debug-name
                                       :system-lambda system-lambda)))
     (setf (functional-inline-expanded clambda) t)
     clambda))
-;;;; defining global functions
-;;; Given a lambda-list, return a FUN-TYPE object representing the signature:
-;;; return type is *, and each individual arguments type is T -- but we get
-;;; the argument counts and keywords.
-;;; TODO: enhance this to optionally accept an alist of (var . type)
-;;; and use that lieu of SB-INTERPRETER:APPROXIMATE-PROTO-FN-TYPE.
-(defun ftype-from-lambda-list (lambda-list)
-  (multiple-value-bind (llks req opt rest key-list)
-      (parse-lambda-list lambda-list :silent t)
-    (flet ((list-of-t (list) (mapcar (constantly t) list)))
-      (let ((reqs (list-of-t req))
-            (opts (when opt (cons '&optional (list-of-t opt))))
-            ;; When it comes to building a type, &REST means pretty much the
-            ;; same thing as &MORE.
-            (rest (when rest '(&rest t)))
-            (keys (when (ll-kwds-keyp llks)
-                    (cons '&key (mapcar (lambda (spec)
-                                          (list (parse-key-arg-spec spec) t))
-                                        key-list))))
-            (allow (when (ll-kwds-allowp llks) '(&allow-other-keys))))
-        (careful-specifier-type `(function (,@reqs ,@opts ,@rest ,@keys ,@allow) *))))))
+
+(declaim (end-block))
 
 ;;; Get a DEFINED-FUN object for a function we are about to define. If
 ;;; the function has been forward referenced, then substitute for the
 ;;; previous references.
 (defun get-defined-fun (name &optional (lambda-list nil lp))
   (proclaim-as-fun-name name)
-  (when (boundp '*ir1-namespace*)
-    (let ((found (find-free-fun name "shouldn't happen! (defined-fun)"))
-          (free-funs (free-funs *ir1-namespace*)))
-      (note-name-defined name :function)
-      (cond ((not (defined-fun-p found))
-             (aver (not (info :function :inlinep name)))
-             (let* ((where-from (leaf-where-from found))
-                    (res (make-defined-fun
-                          :%source-name name
-                          :where-from (if (eq where-from :declared)
-                                          :declared
-                                          :defined-here)
-                          :type (if (eq :declared where-from)
-                                    (leaf-type found)
-                                    (or (and lp
-                                             (ftype-from-lambda-list lambda-list))
-                                        (specifier-type 'function))))))
-               (substitute-leaf res found)
-               (setf (gethash name free-funs) res)))
-            ;; If FREE-FUNS has a previously converted definition
-            ;; for this name, then blow it away and try again.
-            ((defined-fun-functionals found)
-             (remhash name free-funs)
-             (get-defined-fun name lambda-list))
-            (t found)))))
+  (let ((found (find-free-fun name "shouldn't happen! (defined-fun)"))
+        (free-funs (free-funs *ir1-namespace*)))
+    (note-name-defined name :function)
+    (cond ((not (defined-fun-p found))
+           ;; This assertion is wrong in block compilation mode, for
+           ;; instance
+           ;;
+           ;; (defun foo (x) (bar x))
+           ;; (declaim (inline bar))
+           ;; (defun bar (x) x)
+           (aver (or (block-compile *compilation*)
+                     (not (info :function :inlinep name))))
+           (let* ((where-from (leaf-where-from found))
+                  (res (make-defined-fun
+                        :%source-name name
+                        :where-from (if (eq where-from :declared)
+                                        :declared
+                                        :defined-here)
+                        :type (if (eq :declared where-from)
+                                  (leaf-type found)
+                                  (or (and lp
+                                           (ignore-errors
+                                            (ftype-from-lambda-list lambda-list)))
+                                      (specifier-type 'function))))))
+             (substitute-leaf res found)
+             (setf (gethash name free-funs) res)))
+          ;; If FREE-FUNS has a previously converted definition
+          ;; for this name, then blow it away and try again.
+          ((defined-fun-functionals found)
+           (remhash name free-funs)
+           (get-defined-fun name lambda-list))
+          (t found))))
 
 ;;; Check a new global function definition for consistency with
 ;;; previous declaration or definition, and assert argument/result
@@ -1267,14 +1370,30 @@
         (note-inlining (optional-dispatch-more-entry fun))
         (mapc #'note-inlining (optional-dispatch-entry-points fun))))
     ;; substitute for any old references
-    (unless (or (not (block-compile *compilation*))
+    (unless (or (eq (defined-fun-inlinep var) :notinline)
+                (not (block-compile *compilation*))
                 (and info
                      (or (fun-info-transforms info)
                          (fun-info-templates info)
                          (fun-info-ir2-convert info))))
-      (substitute-leaf fun var))
+      (substitute-leaf fun var)
+      ;; If in a simple environment, then we can allow backward references
+      ;; to this function from following top-level forms.
+      (when expansion
+        (push fun (defined-fun-functionals var))))
     fun))
 
+
+;;; Entry point utilities
+
+;;; Return a function for the Nth entry point.
+(defun optional-dispatch-entry-point-fun (dispatcher n)
+  (declare (type optional-dispatch dispatcher)
+           (type unsigned-byte n))
+  (let ((*lexenv* (functional-lexenv dispatcher))
+        (*current-path* (optional-dispatch-source-path dispatcher)))
+    (force (nth n (optional-dispatch-entry-points dispatcher)))))
+
 ;;; Store INLINE-LAMBDA as the inline expansion of NAME.
 ;;; EXTRA-INFO is either a keyword denoting that NAME pertains to
 ;;; an auto-generated defstruct function, or else it is the list of
@@ -1370,9 +1489,9 @@ is potentially harmful to any already-compiled callers using (SAFETY 0)."
 
     (become-defined-fun-name name)
 
-    ;; old CMU CL comment:
-    ;;   If there is a type from a previous definition, blast it,
-    ;;   since it is obsolete.
+    ;;
+    ;; If there is a type from a previous definition, blast it, since it is
+    ;; obsolete.
     (when (and defined-fun (neq :declared (leaf-where-from defined-fun)))
       (setf (leaf-type defined-fun)
             ;; FIXME: If this is a block compilation thing, shouldn't
@@ -1381,39 +1500,3 @@ is potentially harmful to any already-compiled callers using (SAFETY 0)."
             (specifier-type 'function))))
 
   (values))
-
-;; Similar to above, detect duplicate definitions within a file,
-;; but the package lock check is unnecessary - it's handled elsewhere.
-;;
-;; Additionally, this is a STYLE-WARNING, not a WARNING, because there is
-;; meaningful behavior that can be ascribed to some redefinitions, e.g.
-;;  (defmacro foo () first-definition)
-;;  (defun f () (use-it (foo )))
-;;  (defmacro foo () other-definition)
-;; will use the first definition when compiling F, but make the second available
-;; in the loaded fasl. In this usage it would have made sense to wrap the
-;; respective definitions with EVAL-WHEN for different situations,
-;; but as long as the compile-time behavior is deterministic, it's just bad style
-;; and not flat-out wrong, though there is indeed some waste in the fasl.
-;;
-;; KIND is the globaldb KIND of this NAME
-(defun %compiler-defmacro (kind name)
-  (let ((name-key `(,kind ,name)))
-    (when (boundp '*lexenv*)
-      (aver (producing-fasl-file))
-      ;; a slight OAOO issue here wrt %COMPILER-DEFUN
-      (if (member name-key (fun-names-in-this-file *compilation*) :test #'equal)
-          (compiler-style-warn 'same-file-redefinition-warning :name name)
-          (push name-key (fun-names-in-this-file *compilation*))))))
-
-
-;;; Entry point utilities
-
-;;; Return a function for the Nth entry point.
-(defun optional-dispatch-entry-point-fun (dispatcher n)
-  (declare (type optional-dispatch dispatcher)
-           (type unsigned-byte n))
-  (let* ((env (getf (optional-dispatch-plist dispatcher) :ir1-environment))
-         (*lexenv* (first env))
-         (*current-path* (second env)))
-    (force (nth n (optional-dispatch-entry-points dispatcher)))))
