@@ -68,8 +68,8 @@
 (defun instance-structure-protocol-error (slotd fun)
   (error 'instance-structure-protocol-error
          :slotd slotd :fun fun
-         :references (list `(:amop :generic-function ,fun)
-                           '(:amop :section (5 5 3)))))
+         :references `((:amop :generic-function ,fun)
+                       (:amop :section (5 5 3)))))
 
 (defun get-optimized-std-accessor-method-function (class slotd name)
   (cond
@@ -118,7 +118,7 @@
             (check-obsolete-instance instance)
             (let ((value (clos-slots-ref (fsc-instance-slots instance)
                                          location)))
-              (if (eq value +slot-unbound+)
+              (if (unbound-marker-p value)
                   (values
                    (slot-unbound (class-of instance) instance slot-name))
                   value)))
@@ -126,7 +126,7 @@
             (check-obsolete-instance instance)
             (let ((value (clos-slots-ref (std-instance-slots instance)
                                          location)))
-              (if (eq value +slot-unbound+)
+              (if (unbound-marker-p value)
                   (values
                    (slot-unbound (class-of instance) instance slot-name))
                   value)))))
@@ -134,7 +134,7 @@
       (lambda (instance)
         (check-obsolete-instance instance)
         (let ((value (cdr location)))
-          (if (eq value +slot-unbound+)
+          (if (unbound-marker-p value)
               (values (slot-unbound (class-of instance) instance slot-name))
               value))))
      (null
@@ -220,17 +220,15 @@
      (fixnum (if fsc-p
                  (lambda (instance)
                    (check-obsolete-instance instance)
-                   (not (eq (clos-slots-ref (fsc-instance-slots instance)
-                                            location)
-                            +slot-unbound+)))
+                   (not (unbound-marker-p (clos-slots-ref (fsc-instance-slots instance)
+                                                          location))))
                  (lambda (instance)
                    (check-obsolete-instance instance)
-                   (not (eq (clos-slots-ref (std-instance-slots instance)
-                                            location)
-                            +slot-unbound+)))))
+                   (not (unbound-marker-p (clos-slots-ref (std-instance-slots instance)
+                                                          location))))))
      (cons (lambda (instance)
              (check-obsolete-instance instance)
-             (not (eq (cdr location) +slot-unbound+))))
+             (not (unbound-marker-p (cdr location)))))
      (null
       (lambda (instance)
         (declare (ignore instance))
@@ -310,7 +308,7 @@
                     (check-obsolete-instance instance)
                     (let ((value (clos-slots-ref (fsc-instance-slots instance)
                                                  location)))
-                      (if (eq value +slot-unbound+)
+                      (if (unbound-marker-p value)
                           (values (slot-unbound class instance slot-name))
                           value)))
                   (lambda (class instance slotd)
@@ -318,14 +316,14 @@
                     (check-obsolete-instance instance)
                     (let ((value (clos-slots-ref (std-instance-slots instance)
                                                  location)))
-                      (if (eq value +slot-unbound+)
+                      (if (unbound-marker-p value)
                           (values (slot-unbound class instance slot-name))
                           value)))))
       (cons (lambda (class instance slotd)
               (declare (ignore slotd))
               (check-obsolete-instance instance)
               (let ((value (cdr location)))
-                (if (eq value +slot-unbound+)
+                (if (unbound-marker-p value)
                     (values (slot-unbound class instance slot-name))
                     value))))
       (null
@@ -380,17 +378,17 @@
            (lambda (class instance slotd)
              (declare (ignore class slotd))
              (check-obsolete-instance instance)
-             (not (eq (clos-slots-ref (fsc-instance-slots instance) location)
-                      +slot-unbound+)))
+             (not (unbound-marker-p
+                   (clos-slots-ref (fsc-instance-slots instance) location))))
            (lambda (class instance slotd)
              (declare (ignore class slotd))
              (check-obsolete-instance instance)
-             (not (eq (clos-slots-ref (std-instance-slots instance) location)
-                      +slot-unbound+)))))
+             (not (unbound-marker-p
+                   (clos-slots-ref (std-instance-slots instance) location))))))
       (cons (lambda (class instance slotd)
               (declare (ignore class slotd))
               (check-obsolete-instance instance)
-              (not (eq (cdr location) +slot-unbound+))))
+              (not (unbound-marker-p (cdr location)))))
       (null
        (lambda (class instance slotd)
          (declare (ignore class instance))
@@ -551,7 +549,7 @@
 (defun find-slot-cell (wrapper slot-name)
   (declare (symbol slot-name))
   (declare (optimize (sb-c::insert-array-bounds-checks 0)))
-  (let* ((vector (layout-slot-table wrapper))
+  (let* ((vector (wrapper-slot-table wrapper))
          (modulus (truly-the index (svref vector 0)))
          ;; Can elide the 'else' branch of (OR symbol-hash ensure-symbol-hash)
          ;; because every symbol in the slot-table already got a nonzero hash.
