@@ -42,15 +42,6 @@
                          (cell-error-name condition)
                          (type-of (unbound-slot-instance condition))))))))
 
-(defmethod wrapper-fetcher ((class standard-class))
-  '%instance-layout)
-
-(defmethod slots-fetcher ((class standard-class))
-  'std-instance-slots)
-
-(defmethod raw-instance-allocator ((class standard-class))
-  'allocate-standard-instance)
-
 ;;; These three functions work on std-instances and fsc-instances. These are
 ;;; instances for which it is possible to change the wrapper and the slots.
 ;;;
@@ -58,35 +49,6 @@
 ;;; structure protocol are promoted to the implementation-specific class
 ;;; std-class. Many of these methods call these four functions.
 
-(defun %swap-wrappers-and-slots (i1 i2) ; old -> new
-  (cond ((std-instance-p i1)
-         #+(and compact-instance-header x86-64)
-         (let ((oslots (std-instance-slots i1))
-               (nslots (std-instance-slots i2)))
-           ;; The hash val is in the header of the slots. Copying is race-free
-           ;; because it is immutable once memoized by STD-INSTANCE-HASH.
-           (sb-vm::cas-header-data-high
-            nslots 0 (sb-impl::%std-instance-hash oslots)))
-         ;; FIXME: If a backend supports two-word primitive instances
-         ;; and double-wide CAS, it's probably best to use that.
-         ;; Maybe we're inside a mutex here anyway though?
-         (let ((w1 (%instance-layout i1))
-               (s1 (std-instance-slots i1)))
-           (setf (%instance-layout i1) (%instance-layout i2))
-           (setf (std-instance-slots i1) (std-instance-slots i2))
-           (setf (%instance-layout i2) w1)
-           (setf (std-instance-slots i2) s1)))
-        ((fsc-instance-p i1)
-         (let ((w1 (%funcallable-instance-layout i1))
-               (w2 (%funcallable-instance-layout i2))
-               (s1 (fsc-instance-slots i1)))
-           (aver (= (layout-bitmap w1) (layout-bitmap w2)))
-           (setf (%funcallable-instance-layout i1) w2)
-           (setf (fsc-instance-slots i1) (fsc-instance-slots i2))
-           (setf (%funcallable-instance-layout i2) w1)
-           (setf (fsc-instance-slots i2) s1)))
-        (t
-         (error "unrecognized instance type"))))
 
 ;;;; STANDARD-INSTANCE-ACCESS
 
@@ -133,7 +95,7 @@
                  (return-from slot-value
                    (if cell
                        (funcall (slot-info-reader (cdr cell)) object)
-                       (values (slot-missing (wrapper-class* wrapper) object
+                       (values (slot-missing (wrapper-class wrapper) object
                                              slot-name 'slot-value)))))
                 ;; this next test means CONSP, but the transform that weakens
                 ;; CONSP to LISTP isn't working here for some reason.
@@ -142,14 +104,14 @@
                 (t
                  (bug "Bogus slot cell in SLOT-VALUE: ~S" cell)))))
     (if (unbound-marker-p value)
-        (slot-unbound (wrapper-class* wrapper) object slot-name)
+        (slot-unbound (wrapper-class wrapper) object slot-name)
         value)))
 
 (defun set-slot-value (object slot-name new-value)
   (let* ((wrapper (valid-wrapper-of object))
          (cell (or (find-slot-cell wrapper slot-name)
                    (return-from set-slot-value
-                     (progn (slot-missing (wrapper-class* wrapper)
+                     (progn (slot-missing (wrapper-class wrapper)
                                           object slot-name 'setf new-value)
                             new-value))))
          (location (car cell))
@@ -181,7 +143,7 @@
   (let* ((wrapper (valid-wrapper-of object))
          (cell (or (find-slot-cell wrapper slot-name)
                    (return-from slot-value
-                     (values (slot-missing (wrapper-class* wrapper) object slot-name
+                     (values (slot-missing (wrapper-class wrapper) object slot-name
                                            'cas (list old-value new-value))))))
          (location (car cell))
          (info (cdr cell))
@@ -201,7 +163,7 @@
                      (t
                       (bug "Bogus slot-cell in (CAS SLOT-VALUE): ~S" cell)))))
       (if (and (unbound-marker-p old) (neq old old-value))
-          (slot-unbound (wrapper-class* wrapper) object slot-name)
+          (slot-unbound (wrapper-class wrapper) object slot-name)
           old))))
 
 (defun slot-boundp (object slot-name)
@@ -217,7 +179,7 @@
                  (return-from slot-boundp
                    (if cell
                        (funcall (slot-info-boundp (cdr cell)) object)
-                       (and (slot-missing (wrapper-class* wrapper) object
+                       (and (slot-missing (wrapper-class wrapper) object
                                           slot-name 'slot-boundp)
                             t))))
                 ((listp location) ; forcibly transform CONSP to LISTP
@@ -237,10 +199,10 @@
                      +slot-unbound+)))
           ((not location)
            (if cell
-               (let ((class (wrapper-class* wrapper)))
+               (let ((class (wrapper-class wrapper)))
                  (slot-makunbound-using-class class object
                                               (find-slot-definition class slot-name)))
-               (slot-missing (wrapper-class* wrapper) object slot-name
+               (slot-missing (wrapper-class wrapper) object slot-name
                              'slot-makunbound)))
           ((listp location) ; forcibly transform CONSP to LISTP
            (setf (cdr location) +slot-unbound+))
@@ -259,6 +221,11 @@
       (slot-value object slot-name)
       (load-time-value (make-unprintable-object "unbound slot") t)))
 
+(defmacro safely-get-slots (method-name x)
+  `(cond ((std-instance-p ,x) (std-instance-slots ,x))
+         ((fsc-instance-p ,x) (fsc-instance-slots ,x))
+         (t (bug "unrecognized instance type in ~S" ',method-name))))
+
 (defmethod slot-value-using-class ((class std-class)
                                    (object standard-object)
                                    (slotd standard-effective-slot-definition))
@@ -269,14 +236,8 @@
          (value
           (typecase location
             (fixnum
-             (cond ((std-instance-p object)
-                    (clos-slots-ref (std-instance-slots object)
-                                    location))
-                   ((fsc-instance-p object)
-                    (clos-slots-ref (fsc-instance-slots object)
-                                    location))
-                   (t (bug "unrecognized instance type in ~S"
-                           'slot-value-using-class))))
+             (clos-slots-ref (safely-get-slots slot-value-using-class object)
+                             location))
             (cons
              (cdr location))
             (t
@@ -301,14 +262,9 @@
                         new-value)))
     (typecase location
       (fixnum
-       (cond ((std-instance-p object)
-              (setf (clos-slots-ref (std-instance-slots object) location)
-                    new-value))
-             ((fsc-instance-p object)
-              (setf (clos-slots-ref (fsc-instance-slots object) location)
-                    new-value))
-             (t (bug "unrecognized instance type in ~S"
-                     '(setf slot-value-using-class)))))
+       (setf (clos-slots-ref (safely-get-slots (setf slot-value-using-class) object)
+                             location)
+             new-value))
       (cons
        (setf (cdr location) new-value))
       (t
@@ -322,24 +278,17 @@
   ;; FIXME: Do we need this? SLOT-BOUNDP checks for obsolete
   ;; instances. Are users allowed to call this directly?
   (check-obsolete-instance object)
-  (let* ((location (slot-definition-location slotd))
-         (value
+  (let ((location (slot-definition-location slotd)))
+    (not (unbound-marker-p
           (typecase location
             (fixnum
-             (cond ((std-instance-p object)
-                          (clos-slots-ref (std-instance-slots object)
-                                          location))
-                   ((fsc-instance-p object)
-                    (clos-slots-ref (fsc-instance-slots object)
-                                    location))
-                   (t (bug "unrecognized instance type in ~S"
-                           'slot-boundp-using-class))))
+             (clos-slots-ref (safely-get-slots slot-boundp-using-class object)
+                             location))
             (cons
              (cdr location))
             (t
              (instance-structure-protocol-error slotd
-                                                'slot-boundp-using-class)))))
-    (not (unbound-marker-p value))))
+                                                'slot-boundp-using-class)))))))
 
 (defmethod slot-makunbound-using-class
            ((class std-class)
@@ -349,14 +298,9 @@
   (let ((location (slot-definition-location slotd)))
     (typecase location
       (fixnum
-       (cond ((std-instance-p object)
-              (setf (clos-slots-ref (std-instance-slots object) location)
-                    +slot-unbound+))
-             ((fsc-instance-p object)
-              (setf (clos-slots-ref (fsc-instance-slots object) location)
-                    +slot-unbound+))
-             (t (bug "unrecognized instance type in ~S"
-                     'slot-makunbound-using-class))))
+       (setf (clos-slots-ref (safely-get-slots slot-makunbound-using-class object)
+                             location)
+             +slot-unbound+))
       (cons
        (setf (cdr location) +slot-unbound+))
       (t
@@ -469,7 +413,7 @@
        ;; in list. The only exceptions are when there are non-local slots
        ;; before the one we want.
        (slot-definition-name
-        (find position (layout-slot-list (layout-of instance))
+        (find position (wrapper-slot-list (wrapper-of instance))
               :key #'slot-definition-location)))
       (cons
        (car position))))))
