@@ -150,7 +150,7 @@ static inline lispobj canonical_obj(lispobj obj)
 #define check_ptr(index,ptr) if(canonical_obj(ptr)==target) return index;
 static int find_ref(lispobj* source, lispobj target)
 {
-    lispobj layout, bitmap;
+    lispobj layout;
     int scan_limit, i;
 
     lispobj word = *source;
@@ -163,18 +163,16 @@ static int find_ref(lispobj* source, lispobj target)
     scan_limit = sizetab[widetag](source);
     switch (widetag) {
     case INSTANCE_WIDETAG:
-#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
     case FUNCALLABLE_INSTANCE_WIDETAG:
-#endif
-        // mixed boxed/unboxed objects
-        // Unlike in scav_instance where the slot loop is unswitched for
-        // speed into three cases (no raw slots, fixnum bitmap, bignum bitmap),
-        // here we just go for clarity by abstracting out logbitp.
-        layout = instance_layout(source);
+        // Unlike in scav_instance where the slot loop is optimized for
+        // certain special cases, here we opt for simplicity.
+        layout = layout_of(source);
         check_ptr(0, layout);
-        bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
-        for(i=1; i<scan_limit; ++i)
-            if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(i, source[i]);
+        if (layout) {
+            struct bitmap bitmap = get_layout_bitmap(LAYOUT(layout));
+            for(i=1; i<scan_limit; ++i)
+                if (bitmap_logbitp(i-1, bitmap)) check_ptr(i, source[i]);
+        }
         // FIXME: check lockfree_list_node_p() also
         return -1;
 #if FUN_SELF_FIXNUM_TAGGED
@@ -211,7 +209,7 @@ static char* NO_SANITIZE_MEMORY deduce_thread_pc(struct thread* th, void** addr)
     uword_t* fp = __builtin_frame_address(0);
     char* return_pc = 0;
 
-    if (th != arch_os_get_current_thread()) {
+    if (th != get_sb_vm_thread()) {
         int i = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,th));
         os_context_t *c = nth_interrupt_context(i-1, th);
         fp = (uword_t*)*os_context_register_addr(c,reg_FP);
@@ -245,7 +243,7 @@ deduce_thread(void (*context_scanner)(), uword_t pointer, char** pc)
     for_each_thread(th) {
         void **esp=(void **)-1;
         sword_t i,free;
-        if (th == arch_os_get_current_thread())
+        if (th == get_sb_vm_thread())
             esp = (void **)((void *)&pointer);
         else {
             void **esp1;
@@ -352,7 +350,7 @@ static lispobj examine_threads(struct hopscotch_table* targets,
                 *thread_pc = 0;
                 // Scan just the current thread's stack
                 // (We don't know where the other stack pointers are)
-                th = arch_os_get_current_thread();
+                th = get_sb_vm_thread();
                 void **esp = __builtin_frame_address(0);
                 void **where;
                 for (where = ((void **)th->control_stack_end)-1; where >= esp;  --where)
@@ -362,7 +360,7 @@ static lispobj examine_threads(struct hopscotch_table* targets,
                         break;
                     }
             }
-            return pin;
+            if (*root_thread) return pin;
         }
 #endif
     *root_kind = HEAP;
@@ -705,7 +703,7 @@ static boolean ignorep(lispobj* base_ptr,
                        lispobj ignored_objects)
 {
     int i;
-    for (i = fixnum_value(VECTOR(ignored_objects)->length)-1; i >= 0; --i)
+    for (i = vector_len(VECTOR(ignored_objects))-1; i >= 0; --i)
       if (native_pointer(VECTOR(ignored_objects)->data[i]) == base_ptr)
           return 1;
     return 0;
@@ -714,7 +712,7 @@ static boolean ignorep(lispobj* base_ptr,
 static uword_t build_refs(lispobj* where, lispobj* end,
                           struct scan_state* ss)
 {
-    lispobj layout, bitmap;
+    lispobj layout;
     sword_t nwords, scan_limit, i;
     uword_t n_objects = 0, n_scanned_words = 0,
             n_immediates = 0, n_pointers = 0;
@@ -737,19 +735,17 @@ static uword_t build_refs(lispobj* where, lispobj* end,
         nwords = scan_limit = sizetab[widetag](where);
         switch (widetag) {
         case INSTANCE_WIDETAG:
-#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
         case FUNCALLABLE_INSTANCE_WIDETAG:
-#endif
             // mixed boxed/unboxed objects
-            layout = instance_layout(where);
+            layout = layout_of(where);
             check_ptr(layout);
             // Partially initialized instance can't have nonzero words yet
-            bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
-            // If no raw slots, just scan without use of the bitmap.
-            // FIXME: check lockfree_list_node_p() also
-            if (bitmap == make_fixnum(-1)) break;
-            for(i=1; i<scan_limit; ++i)
-                if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(where[i]);
+            if (layout) {
+                struct bitmap bitmap = get_layout_bitmap(LAYOUT(layout));
+                // FIXME: check lockfree_list_node_p() also
+                for(i=1; i<scan_limit; ++i)
+                    if (bitmap_logbitp(i-1, bitmap)) check_ptr(where[i]);
+            }
             continue;
 #if FUN_SELF_FIXNUM_TAGGED
         case CLOSURE_WIDETAG:
@@ -768,13 +764,12 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             // a path from the root only if it does not involve whichever
             // object is definitely weak. This fails on weak key-OR-value
             // tables since we can't decide whether to allow the entry.
-            // Disregard the address-sensitivity bit when checking whether
-            // this is a weak hash-table vector.
-            if ((vector_subtype(*where) & ~subtype_VectorAddrHashing)
-                == subtype_VectorHashing + subtype_VectorWeak) {
+            if (vector_flagp(*where, VectorWeak)) { // if any kind of weak vector
+                // If not a hashtable, just skip it.
+                if (!vector_flagp(*where, VectorHashing)) continue;
                 lispobj* data = where + 2;
-                int kv_vector_length = fixnum_value(where[1]);
-                lispobj lhash_table = data[kv_vector_length-1];
+                int kv_vector_len = vector_len((struct vector*)where);
+                lispobj lhash_table = data[kv_vector_len-1];
                 gc_assert(instancep(lhash_table));
                 struct hash_table* hash_table =
                   (struct hash_table *)native_pointer(lhash_table);
@@ -789,8 +784,6 @@ static uword_t build_refs(lispobj* where, lispobj* end,
                     continue;
                 }
             }
-            if (is_vector_subtype(*where, VectorWeak))
-                continue;
             break;
         default:
             if (!(other_immediate_lowtag_p(widetag) && LOWTAG_FOR_WIDETAG(widetag)))
@@ -885,7 +878,7 @@ static void* compute_heap_inverse(boolean keep_leaves,
     // Add one pointer due to inability to use the first
     // two words of the scratchpad.
     uword_t scratchpad_min_size = (1 + ss.n_pointers) * 2 * sizeof (uint32_t);
-    int pagesize = getpagesize();
+    int pagesize = os_reported_page_size;
     uword_t scratchpad_size = ALIGN_UP(scratchpad_min_size, pagesize);
     ss.scratchpad.base = os_allocate(scratchpad_size);
     gc_assert(ss.scratchpad.base);
@@ -973,7 +966,8 @@ static int trace_paths(void (*context_scanner)(),
     hopscotch_create(&targets, HASH_FUNCTION, 0, 32, 0);
     i = 0;
     do {
-        gc_assert(make_fixnum(i) <= VECTOR(paths)->length);
+        // Oh dear, is this really supposed to be '<=' (vs '<') ?
+        gc_assert(i <= vector_len(VECTOR(paths)));
         lispobj car = CONS(weak_pointers)->car;
         lispobj value = ((struct weak_pointer*)native_pointer(car))->value;
         weak_pointers = CONS(weak_pointers)->cdr;
@@ -1074,20 +1068,3 @@ int prove_liveness(lispobj objects, int criterion)
     extern int gc_n_stack_pins;
     return gc_prove_liveness(0, objects, gc_n_stack_pins, pinned_objects.keys, criterion);
 }
-
-#ifdef LISP_FEATURE_SB_THREAD
-/// Given a pthread identifier, return the lisp thread's string name.
-/// Caveat emptor: This doesn't acquire the all_threads_lock.
-/// Ironically perhaps, 'traceroot' no longer uses this, but it's nice to have.
-struct vector* lisp_thread_name(os_thread_t os_thread)
-{
-    struct thread* th;
-    for_each_thread (th) {
-        if (pthread_equal(os_thread, th->os_thread)) {
-            struct thread_instance *lispthread = (void*)native_pointer(th->lisp_thread);
-            return VECTOR(lispthread->name);
-        }
-    }
-    return 0;
-}
-#endif
